﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.SceneManagement;

public class GameController : MonoBehaviour
{    
    Neuron[,] neurons;
    Neuron start, end;
    List<Neuron> path;
    [HideInInspector]
    public Pulse currentPulse;
    int currentLevel;

    public GameObject menu;
    public int[] levelGridY;
    public Image[] locks;
    public Button[] memoryButtons;
    public GameObject[] memories;
    public Image overlay;
    public GameObject touchBlocker;

    private void Awake()
    {
        path = null;
        Screen.SetResolution(450, 800, false);
    }

    private void Start()
    {        
        int lvl = PlayerPrefs.GetInt("memory",0);
        for (int i = 0; i < locks.Length; i++)
        {
            bool locked = lvl < i;
            locks[i].gameObject.SetActive(locked);
            memoryButtons[i].interactable = !locked;
        }
    }

    public void startLevel(int lvl)
    {
        menu.SetActive(false);
        currentLevel = lvl;
        var grid = ReferenceManager.instance.grid;
        grid.yCount = levelGridY[lvl];
        grid.Refresh();
        generateLevel();
        AudioManager.PlayMenuMusic();
    }



    void generateLevel(){
        createNodes();
        StartCoroutine(emitPulse());
    }


    void createNodes()
    {
        var grid = ReferenceManager.instance.grid;

        neurons = new Neuron[grid.xCount, grid.yCount];

        for (int row = 0; row < grid.yCount; row++)
        {
            for (int col = 0; col < grid.xCount; col++)
            {
                //Get the grid cell rect
                var rect = grid.cells[col, row];
                //Initialize the position with the center of the grid cell
                var pos = rect.center;
                //Add a random jitter to the position
                pos += new Vector2(Random.Range(-rect.size.x, rect.size.x), Random.Range(-rect.size.y, rect.size.y)) / 2;

                var go = Instantiate(ReferenceManager.instance.neuronPrefab);
                go.transform.position = pos;
                var n = go.GetComponent<Neuron>();

                if (col > 0)
                {
                    var left = neurons[col - 1, row];
                    n.SetNeighbour(Direction.Left, left);
                }
                if (col > 0 && row > 0 && Random.Range(0,1f)<0.5f)
                {
                    var bottomLeft = neurons[col - 1, row - 1];
                    n.SetNeighbour(Direction.BottomLeft, bottomLeft);
                }
                if (row > 0)
                {
                    var bottom = neurons[col, row - 1];
                    n.SetNeighbour(Direction.Bottom, bottom);
                }
                if (row > 0 && col < (grid.xCount - 1) && Random.Range(0, 1f) < 0.5f)
                {
                    var bottomRight = neurons[col + 1, row - 1];
                    n.SetNeighbour(Direction.BottomRight, bottomRight);
                }

                n.SetNeuronType(Neuron.NeuronType.Normal);
                neurons[col, row] = n;
            }
        }

        //Assign start and end neurons
        start = neurons[Random.Range(1, grid.xCount - 1), 1]; //First visible row
        start.SetNeuronType(Neuron.NeuronType.Start);

        if (start == null)
            Debug.LogWarning("Start neuron is null!");

        end = neurons[Random.Range(1, grid.xCount - 1), grid.yCount - 2]; //Last visible row
        end.SetNeuronType(Neuron.NeuronType.End);

        if (end == null)
            Debug.LogWarning("End neuron is null!");

        generatePath();
    }




    void generatePath()
    {
        LinkedList<Neuron> lst = new LinkedList<Neuron>();
        var grid = ReferenceManager.instance.grid;

        //Get end neuron
        Neuron n = end;

        for (int row = grid.yCount - 2; row >= 1; row--)
        {
            lst.AddFirst(n);

            //Choose a random down-level neighbour
            var tmp = new List<Neuron>();
            if (row > 1)
            {
                var bl = n.GetNeighbour(Direction.BottomLeft);
                var b = n.GetNeighbour(Direction.Bottom);
                var br = n.GetNeighbour(Direction.BottomRight);
                if (bl != null && bl.GetNeighbour(Direction.Left) != null)
                    tmp.Add(bl);
                if (b != null)
                    tmp.Add(b);
                if (br != null && br.GetNeighbour(Direction.Right) != null)
                    tmp.Add(br);
                if (tmp.Count == 0)
                    break;

                n = tmp[Random.Range(0, tmp.Count)];
            }
            else
            {
                if (n == start)
                    break;
                else if (start.transform.position.x >= end.transform.position.x)
                {
                    //Start point is in the right of current node, go to right
                    while (n != start && n != null)
                    {
                        n = n.GetNeighbour(Direction.Right);
                        if (n != start)
                            lst.AddFirst(n);
                    }
                }
                else
                {
                    //Start point is in the left of current node, go to left
                    while (n != start && n != null)
                    {
                        n = n.GetNeighbour(Direction.Left);
                        if (n != start)
                            lst.AddFirst(n);
                    }
                }
                break;
            }
        }


        lst.AddFirst(start);

        path = new List<Neuron>();
        foreach (var p in lst)
        {
            if (p != null)
                path.Add(p);
            else
            {
                Debug.LogWarning("Neuron in path is null!");
                //FIXME: it shouldn't be null, for now generate path again as a fallback
                generatePath();
                break;
            }
        }
    }

    private void OnDrawGizmos()
    {
        if (path != null)
        {
            Gizmos.color = Color.green;
            for (int i = 1; i < path.Count; i++)
            {
                if (i > 1)
                    Gizmos.color = Color.cyan;
                Gizmos.DrawLine(path[i - 1].transform.position, path[i].transform.position);
            }
        }
    }

    IEnumerator emitPulse(){
        while (true)
        {
            if(currentPulse==null){
                currentPulse = Instantiate(ReferenceManager.instance.pulse.gameObject,start.transform.position,Quaternion.identity).GetComponent<Pulse>();
                generatePath();
                currentPulse.FollowPath(path);
            }

            Color c = ReferenceManager.instance.frontBackground.color;
            StartCoroutine(Utils.DoOverTime(0.2f,(t)=>{
                ReferenceManager.instance.frontBackground.color = Color.Lerp(c,ColorSetter.CodeToColor("412858FF"),t);
                },(
            )=>{
                c = ReferenceManager.instance.frontBackground.color;
                StartCoroutine(Utils.DoOverTime(0.2f,(t)=>{
                    ReferenceManager.instance.frontBackground.color = Color.Lerp(c,ColorSetter.GetColor(ColorPallete.BGColor),t);
                }));
            })); 
            yield return new WaitForSeconds(1.6f); //TODO: hamahang ba ahang
        }
    }


    public void HanldeTouch()
    {
        if(currentPulse!=null)
            currentPulse.Charge();
    }

    public void HandleWin()
    {
        StopAllCoroutines();

        //Dialog.Open("should show memory",new Dialog.ButtonElement("Menu",Reload));
        Color black = Color.black;
        black.a = 0;
        overlay.color = black;

        StartCoroutine(Utils.DoOverTime(1,(t)=>{
            overlay.color = Color.Lerp(black,Color.black,t);
        },()=>{
            touchBlocker.SetActive(false);
            memories[currentLevel].SetActive(true);
            StartCoroutine(Utils.DoOverTime(1,(t)=>{
                overlay.color = Color.Lerp(Color.black,black,t);
            },()=>{
                StartCoroutine(Utils.DoDelayed(5,()=>{touchBlocker.SetActive(true);}));
            }));
        }));

        PlayerPrefs.SetInt("memory", Mathf.Max(PlayerPrefs.GetInt("memory", 0), currentLevel + 1));
        PlayerPrefs.Save();
    }


    public void endOfMemory(){
        Reload();
    }



    public void Reload(){
        SceneManager.LoadScene("mainScene");
    }
}
