﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public static class Utils 
{
    public static IEnumerator MoveItems(Transform[] items, Vector3[] targets, float duration, System.Action onEnd = null, bool smooth = false)
    {
        float t = Time.time;
        float dt = 0;

        Vector3[] starts = new Vector3[items.Length];
        for (int i = 0; i < items.Length; i++)
        {
            starts[i] = items[i].position;
        }

        if (smooth)
        {
            for (; dt < duration; dt = Time.time - t)
            {
                for (int i = 0; i < items.Length; i++)
                {
                    items[i].position = Vector3.Lerp(starts[i], targets[i], Mathf.Sqrt(dt / duration));
                }
                yield return new WaitForEndOfFrame();
            }
        }
        else
        {
            for (; dt < duration; dt = Time.time - t)
            {
                for (int i = 0; i < items.Length; i++)
                {
                    items[i].position = Vector3.Lerp(starts[i], targets[i], (dt / duration));
                }
                yield return new WaitForEndOfFrame();
            }
        }

        for (int i = 0; i < items.Length; i++)
        {
            items[i].position = targets[i];
        }

        if (onEnd != null)
            onEnd();
    }

    public static IEnumerator MoveItem(Transform item, Vector3 target, float duration, System.Action onEnd = null, bool smooth = false)
    {
        float t = Time.time;
        float dt = 0;

        Vector3 start = item.position;

        if (smooth)
        {
            for (; dt < duration; dt = Time.time - t)
            {
                item.position = Vector3.Lerp(start, target, Mathf.Sqrt(dt / duration));                
                yield return new WaitForEndOfFrame();
            }
        }
        else
        {
            for (; dt < duration; dt = Time.time - t)
            {
                item.position = Vector3.Lerp(start, target, (dt / duration));
                yield return new WaitForEndOfFrame();
            }
        }

        item.position = target;        

        if (onEnd != null)
            onEnd();
    }

    public static IEnumerator MoveAndRotateItem(Transform item, Vector3 target, Quaternion targetRotation, float duration, System.Action onEnd = null, bool smooth = false)
    {
        float t = Time.time;
        float dt = 0;

        Vector3 start = item.position;
        Quaternion startRot = item.rotation;
        if (smooth)
        {
            for (; dt < duration; dt = Time.time - t)
            {
                item.position = Vector3.Lerp(start, target, Mathf.Sqrt(dt / duration));
                item.rotation = Quaternion.Lerp(startRot, targetRotation, Mathf.Sqrt(dt / duration));
                yield return new WaitForEndOfFrame();
            }
        }
        else
        {
            for (; dt < duration; dt = Time.time - t)
            {
                item.position = Vector3.Lerp(start, target, (dt / duration));
                item.rotation = Quaternion.Lerp(startRot, targetRotation, (dt / duration));
                yield return new WaitForEndOfFrame();
            }
        }

        item.position = target;
        item.rotation = targetRotation;

        if (onEnd != null)
            onEnd();
    }

	public static IEnumerator RotateItem(Transform item, Quaternion targetRotation, float duration, System.Action onEnd = null, bool smooth = false)
	{
		float t = Time.time;
		float dt = 0;

		Vector3 start = item.position;
		Quaternion startRot = item.rotation;
		if (smooth)
		{
			for (; dt < duration; dt = Time.time - t)
			{
				item.rotation = Quaternion.Lerp(startRot, targetRotation, Mathf.Sqrt(dt / duration));
				yield return new WaitForEndOfFrame();
			}
		}
		else
		{
			for (; dt < duration; dt = Time.time - t)
			{
				item.rotation = Quaternion.Lerp(startRot, targetRotation, (dt / duration));
				yield return new WaitForEndOfFrame();
			}
		}

		item.rotation = targetRotation;

		if (onEnd != null)
			onEnd();
	}

    public static IEnumerator DoDelayed(float delay, System.Action action)
    {
        yield return new WaitForSecondsRealtime(delay);
        action();
    }

    public static IEnumerator ScaleItem(Transform item, Vector3 targetScale, float duration, System.Action onEnd = null, bool smooth = false)
    {
        float t = Time.time;
        float dt = 0;

        Vector3 start = item.localScale;
        if (smooth)
        {
            for (; dt < duration; dt = Time.time - t)
            {
                item.localScale = Vector3.Lerp(start, targetScale, Mathf.Sqrt(dt / duration));
                yield return new WaitForEndOfFrame();
            }
        }
        else
        {
            for (; dt < duration; dt = Time.time - t)
            {
                item.localScale = Vector3.Lerp(start, targetScale, (dt / duration));
                yield return new WaitForEndOfFrame();
            }
        }

        item.localScale = targetScale;

        if (onEnd != null)
            onEnd();
    }

    public static IEnumerator DoOverTime(float duration, System.Action<float> onChange, System.Action onEnd = null, bool smooth = false)
    {
        float t = Time.time;
        float dt = 0;

        if (smooth)
        {
            for (; dt < duration; dt = Time.time - t)
            {
                if (onChange != null)
                    onChange(dt / duration);
                yield return new WaitForEndOfFrame();
            }
        }
        else
        {
            for (; dt < duration; dt = Time.time - t)
            {
                if (onChange != null)
                    onChange(dt / duration);
                yield return new WaitForEndOfFrame();
            }
        }

        if (onEnd != null)
            onEnd();

    }

    public static IEnumerator DoOverUnscaledTime(float duration, System.Action<float> onChange, System.Action onEnd = null, bool smooth = false)
    {
        float t = Time.unscaledTime;
        float dt = 0;

        if (smooth)
        {
            for (; dt < duration; dt = Time.unscaledTime - t)
            {
                if (onChange != null)
                    onChange(dt / duration);
                yield return new WaitForEndOfFrame();
            }
        }
        else
        {
            for (; dt < duration; dt = Time.unscaledTime - t)
            {
                if (onChange != null)
                    onChange(dt / duration);
                yield return new WaitForEndOfFrame();
            }
        }

        if (onEnd != null)
            onEnd();

    }

    public static IEnumerator WaitForSecondsUnscaled(float duration)
    {
        float t = Time.unscaledTime;
        while (Time.unscaledTime - t < duration)
            yield return null;
    }

    public static IEnumerator DoAfterCondition(System.Action action, System.Func<bool> condition)
    {
        while (!condition())
            yield return null;
        action();
    }

    public static IEnumerator DoNextFrame(System.Action action)
    {
        yield return new WaitForEndOfFrame();
        action();
    }

    public static IEnumerator DoAfterFrames(System.Action action, int frameCount)
    {
        for (int i = 0; i < frameCount; i++)
        {
            yield return new WaitForEndOfFrame();
        }
        action();
    }

    public static System.DateTime TimeStampToDateTime(long sec)
    {
        return new System.DateTime(1970, 1, 1).AddSeconds(sec);
    }

    public static System.DateTime TimeStampMillisecondsToDateTime(long ms)
    {
        return new System.DateTime(1970, 1, 1).AddMilliseconds(ms);
    }

    public static long ToTimeStamp(this System.DateTime dt)
    {
        return (long)(dt - new System.DateTime(1970, 1, 1)).TotalSeconds;
    }

    public static long ToTimeStampMilliseconds(this System.DateTime dt)
    {
        return (long)(dt - new System.DateTime(1970, 1, 1)).TotalMilliseconds;
    }

    public static string ToFormattedString(this System.TimeSpan ts, string format)
    {
        var dt = new System.DateTime();
        if (ts.TotalMilliseconds > 0)
            dt = dt.AddMilliseconds(ts.TotalMilliseconds);
        return dt.ToString(format);
    }

    public static void UpdateValue(this IDictionary dct, object key, object val)
    {
        if (dct.Contains(key))
        {
            dct[key] = val;
        }
        else
        {
            dct.Add(key, val);
        }
    }

    public static void UpdateValue(this IDictionary dct, object key, System.Delegate val)
    {
        if (dct.Contains(key))
        {
            dct[key] = val;
        }
        else
        {
            dct.Add(key, val);
        }
    }

    public static object Get(this IDictionary dct, object key, object defaultValue)
    {
        if (dct.Contains(key))
        {
            return dct[key];
        }
        else
        {
            return defaultValue;
        }
    }

    public static void DebugDrawX(Vector2 point, Color? color = null, float? duration = null, float len = 0.5f)
    {
        var p1 = new Vector2(point.x - len, point.y + len);
        var p2 = new Vector2(point.x + len, point.y - len);
        var p3 = new Vector2(point.x + len, point.y + len);
        var p4 = new Vector2(point.x - len, point.y - len);

        if (color != null && duration != null)
        {
            Debug.DrawLine(p1, p2, color.Value, duration.Value);
            Debug.DrawLine(p3, p4, color.Value, duration.Value);
        }
        else if (color != null)
        {
            Debug.DrawLine(p1, p2, color.Value);
            Debug.DrawLine(p3, p4, color.Value);
        }
        else
        {
            Debug.DrawLine(p1, p2);
            Debug.DrawLine(p3, p4);
        }
    }
}